
#if (DEBUG)
// #define SHOWCNT
#endif

using System;


namespace EsbDecimals
{

#if (DEBUG)

	public class DecimalOps
#else
	/// <summary>
	/// Decimal Routines that are missing from the Framework including Sqrt, etc. for <see cref="EsbDecimals"/> v2.1.0 - 10 June
	/// 2011.
	/// </summary>
	public static class DecimalOps
#endif

	{
		#region SignXY Methods

		/// <summary>
		/// Returns the FORTRAN type SIGN of the Values - basically it returns a value
		/// with the Magnitude of First Value and the Sign of the Second Value.
		/// </summary>
		/// <remarks>
		/// If Y is negative then Returns -Abs(X)
		/// else Returns Abs(X)
		/// </remarks>
		/// <param name="X">Value whose magnitude to process.</param>
		/// <param name="Y">Value whose sign to process.</param>
		/// <returns>magnitude of X with sign of Y.</returns>
		public static decimal SignXY(decimal X, decimal Y)
		{
			decimal dX = Math.Abs(X);
			return (Y < 0 ? -dX : dX);
		}

		/// <summary>
		/// Returns the FORTRAN type SIGN of the Values - basically it returns a value
		/// with the Magnitude of First Value and the Sign of the Second Value.
		/// </summary>
		/// <remarks>
		/// If Y is negative then Returns -Abs(X)
		/// else Returns Abs(X)
		/// </remarks>
		/// <param name="X">Value whose magnitude to process.</param>
		/// <param name="Y">Value whose sign to process.</param>
		/// <returns>magnitude of X with sign of Y.</returns>
		public static decimal SignXY(double X, double Y)
		{
			decimal dX = (decimal)Math.Abs(X);
			return (Y < 0 ? -dX : dX);
		}

		/// <summary>
		/// Returns the FORTRAN type SIGN of the Values - basically it returns a value
		/// with the Magnitude of First Value and the Sign of the Second Value.
		/// </summary>
		/// <remarks>
		/// If Y is negative then Returns -Abs(X)
		/// else Returns Abs(X)
		/// </remarks>
		/// <param name="X">Value whose magnitude to process.</param>
		/// <param name="Y">Value whose sign to process.</param>
		/// <returns>magnitude of X with sign of Y.</returns>
		public static decimal SignXY(int X, int Y)
		{
			decimal dX = Math.Abs(X);
			return (Y < 0 ? -dX : dX);
		}

		#endregion //SignXY Methods

		#region Ceiling Methods

		/// <summary>
		/// Returns the Integral Value that is greater than or equal to the specified number.
		/// </summary>
		/// <param name="X">Number to be processed.</param>
		/// <returns>Integral Value that is greater than or equal to the specified number.</returns>
		public static decimal Ceiling(decimal X)
		{
			decimal Result = Decimal.Floor(X);

			if (!CompareOps.SameValue(X, Result))
			{
				Result++;
			}
			return Result;
		}

		/// <summary>
		/// Returns the Integral Value that is greater than or equal to the specified number.
		/// </summary>
		/// <param name="X">Number to be processed.</param>
		/// <returns>Integral Value that is greater than or equal to the specified number.</returns>
		public static decimal Ceiling(double X)
		{
			decimal dX = (decimal)X;
			return Ceiling(dX);
		}

		#endregion //Ceiling Methods

		#region IntPart Methods

		/// <summary>
		/// Returns the Integral portion of the value supplied.
		/// </summary>
		/// <param name="X">Number to be processed.</param>
		/// <returns>Integral portion of the value supplied</returns>
		public static decimal IntPart(decimal X)
		{
			if (X >= 0)
			{
				return Decimal.Floor(X);
			}
			return Ceiling(X);
		}

		/// <summary>
		/// Returns the Integral portion of the value supplied.
		/// </summary>
		/// <param name="X">Number to be processed.</param>
		/// <returns>Integral portion of the value supplied</returns>
		public static decimal IntPart(double X)
		{
			decimal dX = (decimal)X;
			return IntPart(dX);
		}

		#endregion //IntPart Methods

		#region FractionPart Methods

		/// <summary>
		/// Returns the Fractional portion of the value supplied.
		/// </summary>
		/// <param name="X">Number to be processed.</param>
		/// <returns>Fractional portion of the value supplied</returns>
		public static decimal FractionPart(decimal X)
		{
			return X - IntPart(X);
		}

		/// <summary>
		/// Returns the Fractional portion of the value supplied.
		/// </summary>
		/// <param name="X">Number to be processed.</param>
		/// <returns>Fractional portion of the value supplied</returns>
		public static decimal FractionPart(double X)
		{
			decimal dX = (decimal)X;
			return FractionPart(dX);
		}

		#endregion //FractionPart Methods

		#region Powers, Roots and Logarithms

		#region Sqrt Methods

		/// <summary>
		/// Computes the Decimal Square Root using Newton's Method.
		/// </summary>
		/// <exception cref="ArithmeticException">
		/// Thrown when a negative value is passed.
		/// </exception>
		/// <param name="X">We desire the square root of this Value. Must be &gt;= 0</param>
		/// <returns>Square Root of given Value</returns>
		/// <exception cref="ArithmeticException"><c>ArithmeticException</c>.</exception>

		public static decimal Sqrt(decimal X)
		{

			#region Arguments Check

			if (CompareOps.ValueIsNegative(X))
			{
				throw (new ArithmeticException());
			}

			if (CompareOps.ValueIsZero(X))
			{
				return 0.0m;
			}

			#endregion

			const int MaXIterations = 100;
			decimal Guess = 1.0m;
			int Count = 0;

			bool Done = false;

			do
			{
				decimal NewGuess = (X / Guess + Guess) / 2.0m;
				Count++;

				if (Guess == NewGuess)
				{

					Done = true;
				}
				else
				{
					Guess = NewGuess;
				}
			}

			while ((!Done) && (Count < MaXIterations));

			#region Count

#if (SHOWCNT)
			System.Console.WriteLine ("Sqrt Cnt = " + Count.ToString());
#endif

			return Guess;

			#endregion //Count

		}

		/// <summary>
		/// Computes the Decimal Square Root using Newton's Method.
		/// </summary>
		/// <exception cref="ArithmeticException">
		/// Thrown when a negative value is passed.
		/// </exception>
		/// <param name="X">We desire the square root of this Value. Must be &gt;= 0</param>
		/// <returns>Square Root of given Value</returns>
		public static decimal Sqrt(double X)
		{
			decimal dValue = (decimal)X;
			return Sqrt(dValue);
		}

		/// <summary>
		/// Computes the Decimal Square Root using Newton's Method.
		/// </summary>
		/// <exception cref="ArithmeticException">
		/// Thrown when a negative value is passed.
		/// </exception>
		/// <param name="X">We desire the square root of this Value. Must be &gt;= 0</param>
		/// <returns>Square Root of given Value</returns>
		public static decimal Sqrt(int X)
		{
			decimal dValue = X;
			return Sqrt(dValue);
		}

		#endregion //Sqrt Methods

		#region IntPow Methods

		/// <summary>
		/// Computes the value of a Decimal raised to an Integer Power.
		/// </summary>
		/// <param name="X">The Value to be raised to the specified power.</param>
		/// <param name="Y">The Power to raise the specified value to.</param>
		/// <returns>X ^ Y, ie X raised to the power of Y.</returns>
		public static decimal IntPow(decimal X, int Y)
		{

			#region Arguments Check

			if ((CompareOps.ValueIsZero(X)) || (CompareOps.SameValue(X, 1.0m)))
			{
				return X;
			}

			if (CompareOps.SameValue(X, -1.0m))
			{
				if (IntOps.Odd(Y))
				{
					return -1.0m;
				}
				return 1.0m;
			}

			if (Y == 0)
			{
				return 1m;
			}

			#endregion

			decimal Result = 1.0m;

			for (int I = 1; I <= Math.Abs(Y); I++)
			{
				Result = Result * X;
			}

			if (Y < 0)
			{
				return 1.0m / Result;
			}
			return Result;
		}

		/// <summary>
		/// Computes the value of a Decimal raised to an Integer Power.
		/// </summary>
		/// <param name="X">The Value to be raised to the specified power.</param>
		/// <param name="Y">The Power to raise the specified value to.</param>
		/// <returns>X ^ Y, ie X raised to the power of Y.</returns>
		public static decimal IntPow(double X, int Y)
		{
			decimal dX = (decimal)X;
			return IntPow(dX, Y);
		}

		/// <summary>
		/// Computes the value of a Decimal raised to an Integer Power.
		/// </summary>
		/// <param name="X">The Value to be raised to the specified power.</param>
		/// <param name="Y">The Power to raise the specified value to.</param>
		/// <returns>X ^ Y, ie X raised to the power of Y.</returns>
		public static decimal IntPow(int X, int Y)
		{
			decimal dX = X;
			return IntPow(dX, Y);
		}

		#endregion //IntPow Methods

		#region NthRoot Methods

		/// <summary>
		/// Computes the Decimal Nth Root using Newton's Method.
		/// </summary>
		/// <exception cref="ArithmeticException">
		/// Thrown when a negative value is passed or n &lt; 2.
		/// </exception>
		/// <param name="X">We desire the Nth root of this Value. Must be &gt;= 0</param>
		/// <param name="N">The Level of Root - ie Nth Root.</param>
		/// <returns>Nth Root of given Value</returns>
		/// <exception cref="ArithmeticException"><c>ArithmeticException</c>.</exception>
		public static decimal NthRoot(decimal X, int N)
		{

			#region Arguments Check

			if ((CompareOps.ValueIsNegative(X)) || (N < 2))
			{
				throw (new ArithmeticException());
			}

			if (CompareOps.ValueIsZero(X))
			{
				return 0.0m;
			}

			#endregion

			const int MaXIterations = 100;
			decimal Guess = 1;
			int Count = 0;

			bool Done = false;

			do
			{
				decimal NewGuess = (X / IntPow(Guess, N - 1) + (N - 1) * Guess) / N;
				Count++;

				if (Guess == NewGuess)
				{

					Done = true;
				}
				else
				{
					Guess = NewGuess;
				}
			}

			while ((!Done) && (Count < MaXIterations));

			#region Count

#if (SHOWCNT)
			System.Console.WriteLine ("NthRoot Cnt = " + Count.ToString());
#endif

			return Guess;

			#endregion //Count

		}

		/// <summary>
		/// Computes the Decimal Nth Root using Newton's Method.
		/// </summary>
		/// <exception cref="ArithmeticException">
		/// Thrown when a negative value is passed or n &lt; 2.
		/// </exception>
		/// <param name="X">We desire the Nth root of this Value. Must be &gt;= 0</param>
		/// <param name="N">The Level of Root - ie Nth Root.</param>
		/// <returns>Nth Root of given Value</returns>
		public static decimal NthRoot(double X, int N)
		{
			decimal dValue = (decimal)X;
			return NthRoot(dValue, N);
		}

		/// <summary>
		/// Computes the Decimal Nth Root using Newton's Method.
		/// </summary>
		/// <exception cref="ArithmeticException">
		/// Thrown when a negative value is passed or n &lt; 2.
		/// </exception>
		/// <param name="X">We desire the Nth root of this Value. Must be &gt;= 0</param>
		/// <param name="N">The Level of Root - ie Nth Root.</param>
		public static decimal NthRoot(int X, int N)
		{
			decimal dValue = X;
			return NthRoot(dValue, N);
		}

		#endregion //NthRoot Methods

		#region Log Methods

		/// <summary>
		/// Computes the Natural Logarithm using Series Expansion.
		/// </summary>
		/// <exception cref="ArithmeticException">
		/// Thrown when a non-positive value is passed.
		/// </exception>
		/// <param name="X">We desire the natural logarithm of this Value. Must be &gt; 0</param>
		/// <returns>Natural Logarithm of given Value</returns>
		public static decimal Log(decimal X)
		{

			#region Arguments Check

			if (!CompareOps.ValueIsPositive(X))
			{
				throw (new ArithmeticException());
			}

			if (CompareOps.SameValue(X, 1.0m))
			{
				return 0.0m;
			}

			#endregion

			const int MaXIterations = 50000;
			decimal Y = (X - 1) / (X + 1);
			decimal SqrY = Y * Y;
			bool Done = false;
			decimal Result = Y;
			decimal OldResult = Result;
			decimal Z = Y;
			int Cnt = 1;

			do
			{
				Cnt = Cnt + 2;
				Z = Z * SqrY;
				Result = Result + Z / Cnt;

				if (Result == OldResult)
				{

					Done = true;
				}
				else
				{
					OldResult = Result;
				}
			}

			while ((!Done) && (Cnt < MaXIterations));

			#region count

#if (SHOWCNT)
			System.Console.WriteLine ("Log Cnt = " + Cnt.ToString());
#endif

			return 2 * Result;

			#endregion //count

		}

		/// <summary>
		/// Computes the Natural Logarithm using Series Expansion.
		/// </summary>
		/// <exception cref="ArithmeticException">
		/// Thrown when a non-positive value is passed.
		/// </exception>
		/// <param name="X">We desire the natural logarithm of this Value. Must be &gt; 0</param>
		/// <returns>Natural Logarithm of given Value</returns>
		public static decimal Log(double X)
		{
			decimal dX = (decimal)X;
			return Log(dX);
		}

		/// <summary>
		/// Computes the Natural Logarithm using Series Expansion.
		/// </summary>
		/// <exception cref="ArithmeticException">
		/// Thrown when a non-positive value is passed.
		/// </exception>
		/// <param name="X">We desire the natural logarithm of this Value. Must be &gt; 0</param>
		/// <returns>Natural Logarithm of given Value</returns>
		public static decimal Log(int X)
		{
			decimal dX = X;
			return Log(dX);
		}

		#endregion //Log Methods

		#region Log10 Methods


		/// <summary>
		/// Computes the Logarithm to Base 10 using Series Expansion.
		/// </summary>
		/// <exception cref="ArithmeticException">
		/// Thrown when a non-positive value is passed.
		/// </exception>
		/// <param name="X">We desire the logarithm to base 10 of this Value. Must be &gt; 0</param>
		/// <returns>Logarithm to base 10 of given Value</returns>
		public static decimal Log10(decimal X)
		{
			return Log(X) / DecimalGlobals.Ln10;
		}

		/// <summary>
		/// Computes the Logarithm to Base 10 using Series Expansion.
		/// </summary>
		/// <exception cref="ArithmeticException">
		/// Thrown when a non-positive value is passed.
		/// </exception>
		/// <param name="X">We desire the logarithm to base 10 of this Value.Must be &gt; 0</param>
		/// <returns>Logarithm to base 10 of given Value</returns>
		public static decimal Log10(double X)
		{
			decimal dX = (decimal)X;
			return Log10(dX);
		}

		/// <summary>
		/// Computes the Logarithm to Base 10 using Series Expansion.
		/// </summary>
		/// <exception cref="ArithmeticException">
		/// Thrown when a non-positive value is passed.
		/// </exception>
		/// <param name="X">We desire the logarithm to base 10 of this Value. Must be &gt; 0</param>
		/// <returns>Logarithm to base 10 of given Value</returns>
		public static decimal Log10(int X)
		{
			decimal dX = X;
			return Log10(dX);
		}

		#endregion //Log10 Methods

		#region Log2 Methods

		/// <summary>
		/// Computes the Logarithm to Base 2 using Series Expansion.
		/// </summary>
		/// <exception cref="ArithmeticException">
		/// Thrown when a non-positive value is passed.
		/// </exception>
		/// <param name="X">We desire the logarithm to base 2 of this Value. Must be &gt; 0</param>
		/// <returns>Logarithm to base 2 of given Value</returns>
		public static decimal Log2(decimal X)
		{
			return Log(X) / DecimalGlobals.Ln2;
		}

		/// <summary>
		/// Computes the Logarithm to Base 2 using Series Expansion.
		/// </summary>
		/// <exception cref="ArithmeticException">
		/// Thrown when a non-positive value is passed.
		/// </exception>
		/// <param name="X">We desire the logarithm to base 2 of this Value. Must be &gt; 0</param>
		/// <returns>Logarithm to base 2 of given Value</returns>
		public static decimal Log2(double X)
		{
			decimal dX = (decimal)X;
			return Log2(dX);
		}

		/// <summary>
		/// Computes the Logarithm to Base 2 using Series Expansion.
		/// </summary>
		/// <exception cref="ArithmeticException">
		/// Thrown when a non-positive value is passed.
		/// </exception>
		/// <param name="X">We desire the logarithm to base 2 of this Value. Must be &gt; 0</param>
		/// <returns>Logarithm to base 2 of given Value</returns>
		public static decimal Log2(int X)
		{
			decimal dX = X;
			return Log2(dX);
		}

		#endregion //Log2 Methods

		#region Exp Methods

		/// <summary>
		/// Computes the Exponential Function using Series Expansion.
		/// </summary>
		/// <param name="X">Power to raise the Natural Constant, e, to</param>
		/// <returns>e^X, Natural Constant raised to the power X</returns>
		public static decimal Exp(decimal X)
		{

			#region Arguments Check

			if (CompareOps.ValueIsZero(X))
			{
				return 1.0m;
			}

			#endregion

			const int MaXIterations = 50000;
			decimal Y = X;
			bool Done = false;
			decimal Result = 1 + Y;
			decimal OldResult = Result;
			int Cnt = 1;

			do
			{
				Cnt++;
				Y = (Y / Cnt) * X;
				Result = Result + Y;

				if (Result == OldResult)
				{

					Done = true;
				}
				else
				{
					OldResult = Result;
				}
			}

			while ((!Done) && (Cnt < MaXIterations));

			#region Count

#if (SHOWCNT)
			System.Console.WriteLine ("Exp Cnt = " + Cnt.ToString());
#endif

			return Result;

			#endregion //Count

		}

		/// <summary>
		/// Computes the Exponential Function using Series Expansion.
		/// </summary>
		/// <param name="X">Power to raise the Natural Constant, e, to</param>
		/// <returns>e^X, Natural Constant raised to the power X</returns>
		public static decimal Exp(double X)
		{
			decimal dX = (decimal)X;
			return Exp(dX);
		}

		/// <summary>
		/// Computes the Exponential Function using Series Expansion.
		/// </summary>
		/// <param name="X">Power to raise the Natural Constant, e, to</param>
		/// <returns>e^X, Natural Constant raised to the power X</returns>
		public static decimal Exp(int X)
		{
			decimal dX = X;
			return Exp(dX);
		}

		#endregion //Exp Methods

		#region TenPow Methods

		/// <summary>
		/// Computes the Specified Power of 10 using series Expansion.
		/// </summary>
		/// <param name="X">Power to raise 10 to</param>
		/// <returns>10 ^ X, 10 raised to the power X</returns>
		public static decimal TenPow(decimal X)
		{
			return Exp(X * DecimalGlobals.Ln10);
		}

		/// <summary>
		/// Computes the Specified Power of 10 using series Expansion.
		/// </summary>
		/// <param name="X">Power to raise 10 to</param>
		/// <returns>10 ^ X, 10 raised to the power X</returns>
		public static decimal TenPow(double X)
		{
			decimal dX = (decimal)X;
			return TenPow(dX);
		}

		/// <summary>
		/// Computes the Specified Power of 10 using series Expansion.
		/// </summary>
		/// <param name="X">Power to raise 10 to</param>
		/// <returns>10 ^ X, 10 raised to the power X</returns>
		public static decimal TenPow(int X)
		{
			decimal dX = X;
			return TenPow(dX);
		}

		#endregion //TenPow Methods

		#region TwoPow Methods

		/// <summary>
		/// Computes the Specified Power of 2 using series Expansion.
		/// </summary>
		/// <param name="X">Power to raise 2 to</param>
		/// <returns>2 ^ X, 2 raised to the power X</returns>
		public static decimal TwoPow(decimal X)
		{
			return Exp(X * DecimalGlobals.Ln2);
		}

		/// <summary>
		/// Computes the Specified Power of 2 using series Expansion.
		/// </summary>
		/// <param name="X">Power to raise 2 to</param>
		/// <returns>2 ^ X, 2 raised to the power X</returns>
		public static decimal TwoPow(double X)
		{
			decimal dX = (decimal)X;
			return TwoPow(dX);
		}

		/// <summary>
		/// Computes the Specified Power of 2 using series Expansion.
		/// </summary>
		/// <param name="X">Power to raise 2 to</param>
		/// <returns>2 ^ X, 2 raised to the power X</returns>
		public static decimal TwoPow(int X)
		{
			decimal dX = X;
			return TwoPow(dX);
		}

		#endregion //TwoPow Methods

		#region Pow Methods

		/// <summary>
		/// Computes the First Value raised to the power of the Second Value using series Expansion.
		/// </summary>
		/// <exception cref="ArithmeticException">
		/// Thrown when X is Zero and a Negative Power,
		/// or a Negative X is raised to a non-integral power.
		/// </exception>
		/// <param name="X">Value to raise</param>
		/// <param name="Y">Power to use</param>
		/// <returns>X ^ Y, X raised to the power Y</returns>
		public static decimal Pow(decimal X, decimal Y)
		{

			#region Arguments Check

			if (CompareOps.ValueIsZero(Y))
			{
				return 1.0m;
			}

			if (CompareOps.ValueIsZero(X))
			{
				if (Y < 0)
				{
					throw (new ArithmeticException());
				}
				return 0.0m;
			}

			if (CompareOps.SameValue(Y, Decimal.Truncate(Y)))
			{
				return IntPow(X, Decimal.ToInt32(Y));
			}

			if (X < 0)
			{
				throw (new ArithmeticException());
			}

			#endregion

			return Exp(Y * Log(X));
		}

		/// <summary>
		/// Computes the First Value raised to the power of the Second Value using series Expansion.
		/// </summary>
		/// <exception cref="ArithmeticException">
		/// Thrown when X is Zero and a Negative Power,
		/// or a Negative X is raised to a non-integral power.
		/// </exception>
		/// <param name="X">Value to raise</param>
		/// <param name="Y">Power to use</param>
		/// <returns>X ^ Y, X raised to the power Y</returns>
		public static decimal Pow(double X, double Y)
		{
			decimal dX = (decimal)X;
			decimal dY = (decimal)Y;
			return Pow(dX, dY);
		}

		/// <summary>
		/// Computes the First Value raised to the power of the Second Value using series Expansion.
		/// </summary>
		/// <exception cref="ArithmeticException">
		/// Thrown when X is Zero and a Negative Power,
		/// or a Negative X is raised to a non-integral power.
		/// </exception>
		/// <param name="X">Value to raise</param>
		/// <param name="Y">Power to use</param>
		/// <returns>X ^ Y, X raised to the power Y</returns>
		public static decimal Pow(int X, int Y)
		{
			decimal dX = X;
			decimal dY = Y;
			return Pow(dX, dY);
		}

		/// <summary>
		/// Computes the First Value raised to the power of the Second Value using series Expansion.
		/// </summary>
		/// <exception cref="ArithmeticException">
		/// Thrown when X is Zero and a Negative Power,
		/// or a Negative X is raised to a non-integral power.
		/// </exception>
		/// <param name="X">Value to raise</param>
		/// <param name="Y">Power to use</param>
		/// <returns>X ^ Y, X raised to the power Y</returns>
		public static decimal Pow(double X, int Y)
		{
			decimal dX = (decimal)X;
			decimal dY = Y;
			return Pow(dX, dY);
		}

		/// <summary>
		/// Computes the First Value raised to the power of the Second Value using series Expansion.
		/// </summary>
		/// <exception cref="ArithmeticException">
		/// Thrown when X is Zero and a Negative Power,
		/// or a Negative X is raised to a non-integral power.
		/// </exception>
		/// <param name="X">Value to raise</param>
		/// <param name="Y">Power to use</param>
		/// <returns>X ^ Y, X raised to the power Y</returns>
		public static decimal Pow(int X, double Y)
		{
			decimal dX = X;
			decimal dY = (decimal)Y;
			return Pow(dX, dY);
		}

		/// <summary>
		/// Computes the First Value raised to the power of the Second Value using series Expansion.
		/// </summary>
		/// <exception cref="ArithmeticException">
		/// Thrown when X is Zero and a Negative Power,
		/// or a Negative X is raised to a non-integral power.
		/// </exception>
		/// <param name="X">Value to raise</param>
		/// <param name="Y">Power to use</param>
		/// <returns>X ^ Y, X raised to the power Y</returns>
		public static decimal Pow(decimal X, double Y)
		{
			decimal dY = (decimal)Y;
			return Pow(X, dY);
		}

		/// <summary>
		/// Computes the First Value raised to the power of the Second Value using series Expansion.
		/// </summary>
		/// <exception cref="ArithmeticException">
		/// Thrown when X is Zero and a Negative Power,
		/// or a Negative X is raised to a non-integral power.
		/// </exception>
		/// <param name="X">Value to raise</param>
		/// <param name="Y">Power to use</param>
		/// <returns>X ^ Y, X raised to the power Y</returns>
		public static decimal Pow(double X, decimal Y)
		{
			decimal dX = (decimal)X;
			return Pow(dX, Y);
		}

		/// <summary>
		/// Computes the First Value raised to the power of the Second Value using series Expansion.
		/// </summary>
		/// <exception cref="ArithmeticException">
		/// Thrown when X is Zero and a Negative Power,
		/// or a Negative X is raised to a non-integral power.
		/// </exception>
		/// <param name="X">Value to raise</param>
		/// <param name="Y">Power to use</param>
		/// <returns>X ^ Y, X raised to the power Y</returns>
		public static decimal Pow(decimal X, int Y)
		{
			decimal dY = Y;
			return Pow(X, dY);
		}

		/// <summary>
		/// Computes the First Value raised to the power of the Second Value using series Expansion.
		/// </summary>
		/// <exception cref="ArithmeticException">
		/// Thrown when X is Zero and a Negative Power,
		/// or a Negative X is raised to a non-integral power.
		/// </exception>
		/// <param name="X">Value to raise</param>
		/// <param name="Y">Power to use</param>
		/// <returns>X ^ Y, X raised to the power Y</returns>
		public static decimal Pow(int X, decimal Y)
		{
			decimal dX = X;
			return Pow(dX, Y);
		}

		#endregion //Pow Methods

		#endregion //Powers, Roots and Logarithms

		#region Trigonometry

		#region Angle Conversions

		#region AdjustAngleRad Methods

		/// <summary>
		/// Adjust the Angle so that it lies between -Pi (exclusive) and Pi (inclusive)
		/// if <paramref name="PosAngle"/> is False, or between 0 (inclusive) and 2 * Pi (exclusive)
		/// if <paramref name="PosAngle"/> is True.
		/// Works in Radians.
		/// </summary>
		/// <param name="AngleInRadians">Angle in Radians</param>
		/// <param name="PosAngle">When <see langword="true"/> the Angle returned is Positive, ie
		/// between 0 and 2Pi, when <see langword="false"/> then the Angle is between -Pi and Pi.</param>
		/// <returns>Adjusted Angle in Radians</returns>
		public static decimal AdjustAngleRad(decimal AngleInRadians, bool PosAngle)
		{
			decimal Result = AngleInRadians;

			if (!CompareOps.LesserValue(Result, DecimalGlobals.TwoPi))
			{
				Result = Result - Decimal.Floor(Result / DecimalGlobals.TwoPi)
					* DecimalGlobals.TwoPi;
			}

			if (CompareOps.ValueIsNegative(Result))
			{
				Result = Result + (Decimal.Floor(Math.Abs(Result)
					/ DecimalGlobals.TwoPi) + 1.0m) * DecimalGlobals.TwoPi;
			}

			if (PosAngle || (!CompareOps.GreaterValue(Result, DecimalGlobals.Pi)))
			{
				return Result;
			}
			return Result - DecimalGlobals.TwoPi;
		}

		/// <summary>
		/// Adjust the Angle so that it lies between -Pi (exclusive) and Pi (inclusive)
		/// if <paramref name="PosAngle"/> is False, or between 0 (inclusive) and 2 * Pi (exclusive)
		/// if <paramref name="PosAngle"/> is True.
		/// Works in Radians.
		/// </summary>
		/// <param name="AngleInRadians">Angle in Radians</param>
		/// <param name="PosAngle">When <see langword="true"/> the Angle returned is Positive, ie
		/// between 0 and 2Pi, when <see langword="false"/> then the Angle is between -Pi and Pi.</param>
		/// <returns>Adjusted Angle in Radians</returns>
		public static decimal AdjustAngleRad(double AngleInRadians, bool PosAngle)
		{
			decimal dX = (decimal)AngleInRadians;
			return AdjustAngleRad(dX, PosAngle);
		}

		/// <summary>
		/// Adjust the Angle so that it lies between -Pi (exclusive) and Pi (inclusive)
		/// if <paramref name="PosAngle"/> is False, or between 0 (inclusive) and 2 * Pi (exclusive)
		/// if <paramref name="PosAngle"/> is True.
		/// Works in Radians.
		/// </summary>
		/// <param name="AngleInRadians">Angle in Radians</param>
		/// <param name="PosAngle">When <see langword="true"/> the Angle returned is Positive, ie
		/// between 0 and 2Pi, when <see langword="false"/> then the Angle is between -Pi and Pi.</param>
		/// <returns>Adjusted Angle in Radians</returns>
		public static decimal AdjustAngleRad(int AngleInRadians, bool PosAngle)
		{
			decimal dX = AngleInRadians;
			return AdjustAngleRad(dX, PosAngle);
		}

		/// <summary>
		/// Adjust the Angle so that it lies between -Pi (exclusive) and Pi (inclusive)
		/// Works in Radians.
		/// </summary>
		/// <param name="AngleInRadians">Angle in Radians</param>
		/// <returns>Adjusted Angle in Radians</returns>
		public static decimal AdjustAngleRad(decimal AngleInRadians)
		{
			return AdjustAngleRad(AngleInRadians, false);
		}

		/// <summary>
		/// Adjust the Angle so that it lies between -Pi (exclusive) and Pi (inclusive)
		/// Works in Radians.
		/// </summary>
		/// <param name="AngleInRadians">Angle in Radians</param>
		/// <returns>Adjusted Angle in Radians</returns>
		public static decimal AdjustAngleRad(double AngleInRadians)
		{
			decimal dX = (decimal)AngleInRadians;
			return AdjustAngleRad(dX, false);
		}

		/// <summary>
		/// Adjust the Angle so that it lies between -Pi (exclusive) and Pi (inclusive)
		/// Works in Radians.
		/// </summary>
		/// <param name="AngleInRadians">Angle in Radians</param>
		/// <returns>Adjusted Angle in Radians</returns>
		public static decimal AdjustAngleRad(int AngleInRadians)
		{
			decimal dX = AngleInRadians;
			return AdjustAngleRad(dX, false);
		}

		#endregion //AdjustAngleRad Methods

		#region AdjustAngleDeg Methods

		/// <summary>
		/// Adjust the Angle so that it lies between -180 (exclusive) and 180 (inclusive).
		/// if <paramref name="PosAngle"/> is False, or between 0 (inclusive) and 360 (exclusive)
		/// if <paramref name="PosAngle"/> is True.
		/// Works in Degrees.
		/// </summary>
		/// <param name="AngleInDegrees">Angle in Degrees</param>
		/// <param name="PosAngle">When <see langword="true"/> the Angle returned is Positive, ie
		/// between 0 and 360, when <see langword="false"/> then the Angle is between -180 and 180.</param>
		/// <returns>Adjusted Angle in Degrees</returns>
		public static decimal AdjustAngleDeg(decimal AngleInDegrees, bool PosAngle)
		{
			decimal Result = AngleInDegrees;

			if (!CompareOps.LesserValue(Result, 360.0m))
			{
				Result = Result - Decimal.Floor(Result / 360.0m) * 360.0m;
			}

			if (CompareOps.ValueIsNegative(Result))
			{
				Result = Result + (Decimal.Floor(Math.Abs(Result) / 360.0m) + 1.0m) * 360.0m;
			}

			if (PosAngle || (!CompareOps.GreaterValue(Result, 180.0m)))
			{
				return Result;
			}
			return Result - 360.0m;
		}

		/// <summary>
		/// Adjust the Angle so that it lies between -180 (exclusive) and 180 (inclusive)
		/// if <paramref name="PosAngle"/> is False, or between 0 (inclusive) and 360 (exclusive)
		/// if <paramref name="PosAngle"/> is True.
		/// Works in Degrees.
		/// </summary>
		/// <param name="AngleInDegrees">Angle in Degrees</param>
		/// <param name="PosAngle">When True the Angle returned is Positive, ie
		/// between 0 and 360, when False then the Angle is between -180 and 180.</param>
		/// <returns>Adjusted Angle in Degrees</returns>
		public static decimal AdjustAngleDeg(double AngleInDegrees, bool PosAngle)
		{
			decimal dX = (decimal)AngleInDegrees;
			return AdjustAngleDeg(dX, PosAngle);
		}

		/// <summary>
		/// Adjust the Angle so that it lies between -180 (exclusive) and 180 (inclusive)
		/// if <paramref name="PosAngle"/> is False, or between 0 (inclusive) and 360 (exclusive)
		/// if <paramref name="PosAngle"/> is True.
		/// Works in Degrees.
		/// </summary>
		/// <param name="AngleInDegrees">Angle in Degrees</param>
		/// <param name="PosAngle">When True the Angle returned is Positive, ie
		/// between 0 and 360, when False then the Angle is between -180 and 180.</param>
		/// <returns>Adjusted Angle in Degrees</returns>
		public static decimal AdjustAngleDeg(int AngleInDegrees, bool PosAngle)
		{
			decimal dX = AngleInDegrees;
			return AdjustAngleDeg(dX, PosAngle);
		}

		/// <summary>
		/// Adjust the Angle so that it lies between -180 (exclusive) and 180 (inclusive)
		/// Works in Degrees.
		/// </summary>
		/// <param name="AngleInDegrees">Angle in Degrees</param>
		/// <returns>Adjusted Angle in Degrees</returns>
		public static decimal AdjustAngleDeg(decimal AngleInDegrees)
		{
			return AdjustAngleDeg(AngleInDegrees, false);
		}

		/// <summary>
		/// Adjust the Angle so that it lies between -180 (exclusive) and 180 (inclusive)
		/// Works in Degrees.
		/// </summary>
		/// <param name="AngleInDegrees">Angle in Degrees</param>
		/// <returns>Adjusted Angle in Degrees</returns>
		public static decimal AdjustAngleDeg(double AngleInDegrees)
		{
			decimal dX = (decimal)AngleInDegrees;
			return AdjustAngleDeg(dX, false);
		}

		/// <summary>
		/// Adjust the Angle so that it lies between -180 (exclusive) and 180 (inclusive)
		/// Works in Degrees.
		/// </summary>
		/// <param name="AngleInDegrees">Angle in Degrees</param>
		/// <returns>Adjusted Angle in Degrees</returns>
		public static decimal AdjustAngleDeg(int AngleInDegrees)
		{
			decimal dX = AngleInDegrees;
			return AdjustAngleDeg(dX, false);
		}

		#endregion //AdjustAngleDeg Methods

		#region Deg2Rad Methods

		/// <summary>
		/// Converts Degrees into Radians.
		/// </summary>
		/// <param name="AngleInDegrees">Angle in Degrees</param>
		/// <returns>Angle In Radians</returns>
		public static decimal Deg2Rad(decimal AngleInDegrees)
		{
			return AngleInDegrees * DecimalGlobals.OneDegree;
		}

		/// <summary>
		/// Converts Degrees into Radians.
		/// </summary>
		/// <param name="AngleInDegrees">Angle in Degrees</param>
		/// <returns>Angle In Radians</returns>
		public static decimal Deg2Rad(double AngleInDegrees)
		{
			decimal dX = (decimal)AngleInDegrees;
			return dX * DecimalGlobals.OneDegree;
		}

		/// <summary>
		/// Converts Degrees into Radians.
		/// </summary>
		/// <param name="AngleInDegrees">Angle in Degrees</param>
		/// <returns>Angle In Radians</returns>
		public static decimal Deg2Rad(int AngleInDegrees)
		{
			decimal dX = AngleInDegrees;
			return dX * DecimalGlobals.OneDegree;
		}

		#endregion //Deg2Rad Methods

		#region Rad2Deg Methods

		/// <summary>
		/// Converts Radians into Degrees.
		/// </summary>
		/// <param name="AngleInRadians">Angle in Radians</param>
		/// <returns>Angle in Degrees</returns>
		public static decimal Rad2Deg(double AngleInRadians)
		{
			decimal dX = (decimal)AngleInRadians;
			return dX * DecimalGlobals.OneRadian;
		}

		/// <summary>
		/// Converts Radians into Degrees.
		/// </summary>
		/// <param name="AngleInRadians">Angle in Radians</param>
		/// <returns>Angle in Degrees</returns>
		public static decimal Rad2Deg(int AngleInRadians)
		{
			decimal dX = AngleInRadians;
			return dX * DecimalGlobals.OneRadian;
		}

		/// <summary>
		/// Converts Radians into Degrees.
		/// </summary>
		/// <param name="AngleInRadians">Angle in Radians</param>
		/// <returns>Angle in Degrees</returns>
		public static decimal Rad2Deg(decimal AngleInRadians)
		{
			return AngleInRadians * DecimalGlobals.OneRadian;
		}

		#endregion //Rad2Deg Methods

		#region Grad2Rad Methods

		/// <summary>
		/// Converts Grads into Radians.
		/// </summary>
		/// <param name="AngleInGrads">Angle in Grads</param>
		/// <returns>Angle In Radians</returns>
		public static decimal Grad2Rad(decimal AngleInGrads)
		{
			return AngleInGrads * DecimalGlobals.Pi * 0.005m;
		}

		/// <summary>
		/// Converts Grads into Radians.
		/// </summary>
		/// <param name="AngleInGrads">Angle in Grads</param>
		/// <returns>Angle In Radians</returns>
		public static decimal Grad2Rad(double AngleInGrads)
		{
			decimal dX = (decimal)AngleInGrads;
			return Grad2Rad(dX);
		}

		/// <summary>
		/// Converts Grads into Radians.
		/// </summary>
		/// <param name="AngleInGrads">Angle in Grads</param>
		/// <returns>Angle In Radians</returns>
		public static decimal Grad2Rad(int AngleInGrads)
		{
			decimal dX = AngleInGrads;
			return Grad2Rad(dX);
		}

		#endregion //Grad2Rad Methods

		#region Rad2Grad Methods

		/// <summary>
		/// Converts Radians into Grads.
		/// </summary>
		/// <param name="AngleInRadians">Angle in Radians</param>
		/// <returns>Angle in Grads</returns>
		public static decimal Rad2Grad(decimal AngleInRadians)
		{
			return (200.0m * DecimalGlobals.InvPi) * AngleInRadians;
		}

		/// <summary>
		/// Converts Radians into Grads.
		/// </summary>
		/// <param name="AngleInRadians">Angle in Radians</param>
		/// <returns>Angle in Grads</returns>
		public static decimal Rad2Grad(double AngleInRadians)
		{
			decimal dX = (decimal)AngleInRadians;
			return Rad2Grad(dX);
		}

		/// <summary>
		/// Converts Radians into Grads.
		/// </summary>
		/// <param name="AngleInRadians">Angle in Radians</param>
		/// <returns>Angle in Grads</returns>
		public static decimal Rad2Grad(int AngleInRadians)
		{
			decimal dX = AngleInRadians;
			return Rad2Grad(dX);
		}

		#endregion //Rad2Grad Methods

		#region Grad2Deg Methods

		/// <summary>
		/// Converts Grads into Degrees.
		/// </summary>
		/// <param name="AngleInGrads">Angle in Grads</param>
		/// <returns>Angle In Degrees</returns>
		public static decimal Grad2Deg(decimal AngleInGrads)
		{
			return AngleInGrads * 0.9m;
		}

		/// <summary>
		/// Converts Grads into Degrees.
		/// </summary>
		/// <param name="AngleInGrads">Angle in Grads</param>
		/// <returns>Angle In Degrees</returns>
		public static decimal Grad2Deg(double AngleInGrads)
		{
			decimal dX = (decimal)AngleInGrads;
			return Grad2Deg(dX);
		}

		/// <summary>
		/// Converts Grads into Degrees.
		/// </summary>
		/// <param name="AngleInGrads">Angle in Grads</param>
		/// <returns>Angle In Degrees</returns>
		public static decimal Grad2Deg(int AngleInGrads)
		{
			decimal dX = AngleInGrads;
			return Grad2Deg(dX);
		}

		#endregion //Grad2Deg Methods

		#region Deg2Grad Methods

		/// <summary>
		/// Converts Degrees into Grads.
		/// </summary>
		/// <param name="AngleInDegrees">Angle in Degrees</param>
		/// <returns>Angle in Grads</returns>
		public static decimal Deg2Grad(decimal AngleInDegrees)
		{
			return AngleInDegrees / 0.9m;
		}

		/// <summary>
		/// Converts Degrees into Grads.
		/// </summary>
		/// <param name="AngleInDegrees">Angle in Degrees</param>
		/// <returns>Angle in Grads</returns>
		public static decimal Deg2Grad(double AngleInDegrees)
		{
			decimal dX = (decimal)AngleInDegrees;
			return Deg2Grad(dX);
		}

		/// <summary>
		/// Converts Degrees into Grads.
		/// </summary>
		/// <param name="AngleInDegrees">Angle in Degrees</param>
		/// <returns>Angle in Grads</returns>
		public static decimal Deg2Grad(int AngleInDegrees)
		{
			decimal dX = AngleInDegrees;
			return Deg2Grad(dX);
		}

		#endregion //Deg2Grad Methods

		#region Dms2Deg Methods

		/// <summary>
		/// Converts Degrees/Minutes/Seconds into Decimal Degrees. 
		/// </summary>
		/// <param name="varDegrees">Number of Degrees.</param>
		/// <param name="varMinutes">Number of Minutes.</param>
		/// <param name="varSeconds">Number of Seconds.</param>
		/// <returns> </returns>
		public static decimal Dms2Deg(decimal varDegrees, decimal varMinutes, decimal varSeconds)
		{
			return varDegrees + varMinutes / 60 + varSeconds / 3600;
		}

		/// <summary>
		/// Converts Degrees/Minutes/Seconds into Decimal Degrees. 
		/// </summary>
		/// <param name="varDegrees">Number of Degrees.</param>
		/// <param name="varMinutes">Number of Minutes.</param>
		/// <param name="varSeconds">Number of Seconds.</param>
		/// <returns> </returns>
		public static decimal Dms2Deg(double varDegrees, double varMinutes, double varSeconds)
		{
			decimal Degrees = (decimal)varDegrees;
			decimal Minutes = (decimal)varMinutes;
			decimal Seconds = (decimal)varSeconds;
			return Dms2Deg(Degrees, Minutes, Seconds);
		}

		/// <summary>
		/// Converts Degrees/Minutes/Seconds into Decimal Degrees. 
		/// </summary>
		/// <param name="varDegrees">Number of Degrees.</param>
		/// <param name="varMinutes">Number of Minutes.</param>
		/// <param name="varSeconds">Number of Seconds.</param>
		/// <returns> </returns>
		public static decimal Dms2Deg(int varDegrees, int varMinutes, int varSeconds)
		{
			decimal Degrees = varDegrees;
			decimal Minutes = varMinutes;
			decimal Seconds = varSeconds;
			return Dms2Deg(Degrees, Minutes, Seconds);
		}

		#endregion //Dms2Deg Methods

		#region Deg2Dms Methods

		/// <summary>
		/// Converts a Decimal Degree into Degrees/Minutes/Seconds. Returned values are
		/// positive with Sign containing the <see cref="System.Math.Sign(decimal)"/> of the original
		/// value.
		/// </summary>
		/// <param name="decDegrees">Degrees in decimal format. Can be Negative.</param>
		/// <param name="varDegrees">Whole Degrees, &gt;= 0</param>
		/// <param name="varMinutes">Whole Minutes, &gt;= 0</param>
		/// <param name="varSeconds">Seconds, including fractional portion, &gt;= 0</param>
		/// <param name="varSign">1 if &gt; 0, 0 if = 0, -1 if &lt; 0</param>
		/// <returns> </returns>
		public static void Deg2Dms(decimal decDegrees, out int varDegrees, out int varMinutes,
			out decimal varSeconds, out int varSign)
		{
			varSign = Math.Sign(decDegrees);
			decimal X = Math.Abs(decDegrees);
			varDegrees = IntOps.Floor(X);
			X = FractionPart(X) * 60;
			varMinutes = IntOps.Floor(X);
			varSeconds = FractionPart(X) * 60;
		}

		/// <summary>
		/// Converts a Decimal Degree into Degrees/Minutes/Seconds. Returned values are
		/// positive with Sign containing the <see cref="System.Math.Sign(decimal)"/> of the original
		/// value.
		/// </summary>
		/// <param name="decDegrees">Degrees in decimal format. Can be Negative.</param>
		/// <param name="varDegrees">Whole Degrees, &gt;= 0</param>
		/// <param name="varMinutes">Whole Minutes, &gt;= 0</param>
		/// <param name="varSeconds">Seconds, including fractional portion, &gt;= 0</param>
		/// <param name="varSign">1 if &gt; 0, 0 if = 0, -1 if &lt; 0</param>
		/// <returns> </returns>
		public static void Deg2Dms(double decDegrees, out int varDegrees, out int varMinutes,
			out decimal varSeconds, out int varSign)
		{
			decimal dX = (decimal)decDegrees;
			Deg2Dms(dX, out varDegrees, out varMinutes, out varSeconds, out varSign);
		}

		#endregion //Deg2Dms Methods

		#endregion //Angle Conversions

		#region Radian Trig Ops

		#region Sin Methods

		/// <summary>
		/// <para>Computes the Sine of the given angle (in Radians) using Series Expansion.</para>
		/// </summary>
		/// <param name="AngleInRadians">Angle in Radians</param>
		/// <returns>Sine of the Angle</returns>
		public static decimal Sin(decimal AngleInRadians)
		{
			decimal Angle = AdjustAngleRad(AngleInRadians);

			#region Arguments Check

			if (CompareOps.ValueIsZero(Angle))
			{
				return 0.0m;
			}

			#endregion

			const int MaXIterations = 50000;
			decimal Y = Angle;
			bool Done = false;
			bool Negative = false;
			decimal Result = Y;
			decimal OldResult = Result;
			int Cnt = 1;

			do
			{
				Negative = !Negative;
				Cnt = Cnt + 2;
				Y = (Y / (Cnt * (Cnt - 1))) * Angle * Angle;

				if (Negative)
				{
					Result = Result - Y;
				}
				else
				{
					Result = Result + Y;
				}

				if (Result == OldResult)
				{

					Done = true;
				}
				else
				{
					OldResult = Result;
				}
			}

			while ((!Done) && (Cnt < MaXIterations));

			#region Count

#if (SHOWCNT)
			System.Console.WriteLine ("Sin Cnt = " + Cnt.ToString());
#endif

			return Result;

			#endregion //Count

		}

		/// <summary>
		/// <para>Computes the Sine of the given angle (in Radians) using Series Expansion.</para>
		/// </summary>
		/// <param name="AngleInRadians">Angle in Radians</param>
		/// <returns>Sine of the Angle</returns>
		public static decimal Sin(double AngleInRadians)
		{
			decimal dX = (decimal)AngleInRadians;
			return Sin(dX);
		}

		/// <summary>
		/// <para>Computes the Sine of the given angle (in Radians) using Series Expansion.</para>
		/// </summary>
		/// <param name="AngleInRadians">Angle in Radians</param>
		/// <returns>Sine of the Angle</returns>
		public static decimal Sin(int AngleInRadians)
		{
			decimal dX = AngleInRadians;
			return Sin(dX);
		}

		#endregion //Sin Methods

		#region Cos Methods

		/// <summary>
		/// <para>Computes the Cosine of the given angle (in Radians) using Series Expansion.</para>
		/// </summary>
		/// <param name="AngleInRadians">Angle in Radians</param>
		/// <returns>Cosine of the Angle</returns>
		public static decimal Cos(decimal AngleInRadians)
		{
			decimal Angle = AdjustAngleRad(AngleInRadians);

			#region Arguments Check

			if (CompareOps.ValueIsZero(Angle))
			{
				return 1.0m;
			}

			#endregion

			const int MaXIterations = 50000;
			decimal Y = 1.0m;
			bool Done = false;
			bool Negative = false;
			decimal Result = Y;
			decimal OldResult = Result;
			int Cnt = 0;

			do
			{
				Negative = !Negative;
				Cnt = Cnt + 2;
				Y = (Y / (Cnt * (Cnt - 1))) * Angle * Angle;

				if (Negative)
				{
					Result = Result - Y;
				}
				else
				{
					Result = Result + Y;
				}

				if (Result == OldResult)
				{

					Done = true;
				}
				else
				{
					OldResult = Result;
				}
			}

			while ((!Done) && (Cnt < MaXIterations));

			#region Count

#if (SHOWCNT)
			System.Console.WriteLine ("Cos Cnt = " + Cnt.ToString());
#endif

			return Result;

			#endregion //Count

		}

		/// <summary>
		/// <para>Computes the Cosine of the given angle (in Radians) using Series Expansion.</para>
		/// </summary>
		/// <param name="AngleInRadians">Angle in Radians</param>
		/// <returns>Cosine of the Angle</returns>
		public static decimal Cos(double AngleInRadians)
		{
			decimal dX = (decimal)AngleInRadians;
			return Cos(dX);
		}

		/// <summary>
		/// <para>Computes the Cosine of the given angle (in Radians) using Series Expansion.</para>
		/// </summary>
		/// <param name="AngleInRadians">Angle in Radians</param>
		/// <returns>Cosine of the Angle</returns>
		public static decimal Cos(int AngleInRadians)
		{
			decimal dX = AngleInRadians;
			return Cos(dX);
		}

		#endregion //Cos Methods

		#region SinCos Methods

		/// <summary>
		/// Computes the Sine and Cosine of the given angle (in Radians) using Series Expansion.
		/// </summary>
		/// <param name="AngleInRadians">Angle in Radians</param>
		/// <param name="SinX">Sine of the Angle</param>
		/// <param name="CosX">Cosine of the Angle</param>
		public static void SinCos(decimal AngleInRadians, out decimal SinX, out decimal CosX)
		{
			decimal Angle = AdjustAngleRad(AngleInRadians);

			#region Arguments Check

			if (CompareOps.ValueIsZero(Angle))
			{
				SinX = 0.0m;
				CosX = 1.0m;
				return;
			}

			#endregion

			const int MaXIterations = 50000;
			decimal Y = Angle;
			bool Done = false;
			CosX = 1.0m;
			SinX = Y;
			decimal OldsinX = SinX;
			decimal OldcosX = CosX;
			int Cnt = 1;

			do
			{
				Cnt++;
				bool Negative = IntOps.Odd(Cnt / 2);
				Y = (Y / Cnt) * Angle;

				if (Negative)
				{
					CosX = CosX - Y;
				}
				else
				{
					CosX = CosX + Y;
				}
				Cnt++;
				Y = (Y / Cnt) * Angle;

				if (Negative)
				{
					SinX = SinX - Y;
				}
				else
				{
					SinX = SinX + Y;
				}

				if (CompareOps.SameValue(SinX, OldsinX) && CompareOps.SameValue(CosX, OldcosX))
				{

					Done = true;
				}
				else
				{
					OldsinX = SinX;
					OldcosX = CosX;
				}
			}

			while ((!Done) && (Cnt < MaXIterations));

			#region Count

#if (SHOWCNT)
			System.Console.WriteLine ("SinCos Cnt = " + Cnt.ToString());
#endif

			return;

			#endregion //Count

		}

		/// <summary>
		/// Computes the Sine and Cosine of the given angle (in Radians) using Series Expansion.
		/// </summary>
		/// <param name="AngleInRadians">Angle in Radians</param>
		/// <param name="SinX">Sine of the Angle</param>
		/// <param name="CosX">Cosine of the Angle</param>
		public static void SinCos(double AngleInRadians, out decimal SinX, out decimal CosX)
		{
			decimal dX = (decimal)AngleInRadians;
			SinCos(dX, out SinX, out CosX);
			return;
		}

		/// <summary>
		/// Computes the Sine and Cosine of the given angle (in Radians) using Series Expansion.
		/// </summary>
		/// <param name="AngleInRadians">Angle in Radians</param>
		/// <param name="SinX">Sine of the Angle</param>
		/// <param name="CosX">Cosine of the Angle</param>
		public static void SinCos(int AngleInRadians, out decimal SinX, out decimal CosX)
		{
			decimal dX = AngleInRadians;
			SinCos(dX, out SinX, out CosX);
			return;
		}

		#endregion //SinCos Methods

		#region Tan Methods

		/// <summary>
		/// Computes the Tangent of the given angle (in Radians).
		/// </summary>
		/// <param name="AngleInRadians">Angle in Radians</param>
		/// <returns>Tangent of the Angle</returns>
		/// <exception cref="DivideByZeroException">
		/// Thrown when the angle falls on the Pi/2 or 3*Pi/2 Axes.
		/// </exception>
		public static decimal Tan(decimal AngleInRadians)
		{
			decimal SinX;
			decimal CosX;
			SinCos(AngleInRadians, out SinX, out CosX);

			if (CompareOps.ValueIsZero(CosX))
			{
				throw (new DivideByZeroException());
			}
			return SinX / CosX;
		}

		/// <summary>
		/// Computes the Tangent of the given angle (in Radians).
		/// </summary>
		/// <param name="AngleInRadians">Angle in Radians</param>
		/// <returns>Tangent of the Angle</returns>
		/// <exception cref="DivideByZeroException">
		/// Thrown when the angle falls on the Pi/2 or 3*Pi/2 Axes.
		/// </exception>
		public static decimal Tan(double AngleInRadians)
		{
			decimal dX = (decimal)AngleInRadians;
			return Tan(dX);
		}

		/// <summary>
		/// Computes the Tangent of the given angle (in Radians).
		/// </summary>
		/// <param name="AngleInRadians">Angle in Radians</param>
		/// <returns>Tangent of the Angle</returns>
		/// <exception cref="DivideByZeroException">
		/// Thrown when the angle falls on the Pi/2 or 3*Pi/2 Axes.
		/// </exception>
		public static decimal Tan(int AngleInRadians)
		{
			decimal dX = AngleInRadians;
			return Tan(dX);
		}

		#endregion //Tan Methods

		#region Asin Methods

		/// <summary>
		/// Computes the angle (in Radians) that resulted in this Sine Value using Series Expansion.
		/// Thus computing the inverse sine or arcsin.
		/// </summary>
		/// <param name="X">Sine of the Angle</param>
		/// <returns>Angle in Radians in the range of -Pi/2 to Pi/2 inclusive.</returns>
		/// <exception cref="ArithmeticException">
		/// Thrown when abs (Value) &gt; 1.
		/// </exception>
		public static decimal Asin(decimal X)
		{

			#region Arguments Check

			if (CompareOps.SameValue(X, 1.0m))
			{
				return DecimalGlobals.PiOver2;
			}

			if (CompareOps.SameValue(X, -1.0m))
			{
				return -DecimalGlobals.PiOver2;
			}

			if (CompareOps.ValueIsZero(X))
			{
				return 0.0m;
			}

			if ((CompareOps.GreaterValue(X, 1.0m) || CompareOps.LesserValue(X, -1.0m)))
			{
				throw (new ArithmeticException());
			}

			#endregion

			const int MaXIterations = 50000;
			decimal Y = X;
			bool Done = false;
			decimal Result = Y;
			decimal OldResult = Result;
			int Cnt = 1;

			do
			{
				Cnt = Cnt + 2;
				Y = (Y / (Cnt - 1)) * (Cnt - 2) * X * X;
				Result = Result + Y / Cnt;

				if (Result == OldResult)
				{

					Done = true;
				}
				else
				{
					OldResult = Result;
				}
			}

			while ((!Done) && (Cnt < MaXIterations));

			#region Count

#if (SHOWCNT)
			System.Console.WriteLine ("Asin Cnt = " + Cnt.ToString());
#endif

			return Result;

			#endregion //Count

		}

		/// <summary>
		/// Computes the angle (in Radians) that resulted in this Sine Value using Series Expansion.
		/// Thus computing the inverse sine or arcsin.
		/// </summary>
		/// <param name="X">Sine of the Angle</param>
		/// <returns>Angle in Radians in the range of -Pi/2 to Pi/2 inclusive.</returns>
		/// <exception cref="ArithmeticException">
		/// Thrown when abs (Value) &gt; 1.
		/// </exception>
		public static decimal Asin(double X)
		{
			decimal dX = (decimal)X;
			return Asin(dX);
		}

		#endregion //Asin Methods

		#region Acos Methods

		/// <summary>
		/// Computes the angle (in Radians) that resulted in this Cosine Value using <see cref="Asin(decimal)"/>.
		/// Thus computing the inverse cosine or arccos.
		/// </summary>
		/// <param name="X">Cosine of the Angle</param>
		/// <returns>Angle in Radians in the range of 0 to Pi inclusive.</returns>
		/// <exception cref="ArithmeticException">
		/// Thrown when abs (Value) &gt; 1.
		/// </exception>
		public static decimal Acos(decimal X)
		{

			#region Arguments Check

			if (CompareOps.SameValue(X, 1.0m))
			{
				return 0.0m;
			}

			if (CompareOps.SameValue(X, -1.0m))
			{
				return DecimalGlobals.Pi;
			}

			if (CompareOps.ValueIsZero(X))
			{
				return DecimalGlobals.PiOver2;
			}

			if ((CompareOps.GreaterValue(X, 1.0m) || CompareOps.LesserValue(X, -1.0m)))
			{
				throw (new ArithmeticException());
			}

			#endregion

			return DecimalGlobals.PiOver2 - Asin(X);
		}

		/// <summary>
		/// Computes the angle (in Radians) that resulted in this Cosine Value using <see cref="Asin(decimal)"/>.
		/// Thus computing the inverse cosine or arccos.
		/// </summary>
		/// <param name="X">Cosine of the Angle</param>
		/// <returns>Angle in Radians in the range of 0 to Pi inclusive.</returns>
		/// <exception cref="ArithmeticException">
		/// Thrown when abs (Value) &gt; 1.
		/// </exception>
		public static decimal Acos(double X)
		{
			decimal dX = (decimal)X;
			return Acos(dX);
		}

		#endregion //Acos Methods

		#region Atan Methods

		private static decimal AtanLt1(decimal X)
		{
			const int MaXIterations = 50000;
			int Cnt = 1;
			decimal Y = X;
			bool Done = false;
			bool Negative = false;
			decimal Result = Y;
			decimal OldResult = Result;

			do
			{
				Negative = !Negative;
				Cnt = Cnt + 2;
				Y = Y * X * X;

				if (Negative)
				{
					Result = Result - Y / Cnt;
				}
				else
				{
					Result = Result + Y / Cnt;
				}

				if (Result == OldResult)
				{

					Done = true;
				}
				else
				{
					OldResult = Result;
				}
			}

			while ((!Done) && (Cnt < MaXIterations));

			#region Count

#if (SHOWCNT)
			System.Console.WriteLine ("Atan Cnt = " + Cnt.ToString());
#endif

			return Result;

			#endregion //Count

		}

		/// <summary>
		/// <para>Computes the angle (in Radians) that resulted in this Tangent Value using Series Expansion.
		/// Thus computing the inverse tangent or arctan.</para>
		/// <para>If you want to cover all quadrants then you may want to use <see cref="Atan(decimal)"/></para>
		/// </summary>
		/// <param name="X">Tangent of the Angle</param>
		/// <returns>Angle in Radians in the range of -Pi/2 to Pi/2 inclusive.</returns>
		public static decimal Atan(decimal X)
		{

			#region Arguments Check

			if (CompareOps.SameValue(X, 1.0m))
			{
				return DecimalGlobals.PiOver4;
			}

			if (CompareOps.SameValue(X, -1.0m))
			{
				return -DecimalGlobals.PiOver4;
			}

			if (CompareOps.ValueIsZero(X))
			{
				return 0.0m;
			}

			#endregion

			if (Math.Abs(X) < 1.0m)
			{
				return AtanLt1(X);
			}
			if (X > 0.0m)
			{
				return DecimalGlobals.PiOver2 - AtanLt1(1 / X);
			}
			return -(DecimalGlobals.PiOver2 - AtanLt1(1 / (-X)));
		}

		/// <summary>
		/// <para>Computes the angle (in Radians) that resulted in this Tangent Value using Series Expansion.
		/// Thus computing the inverse tangent or arctan.</para>
		/// <para>If you want to cover all quadrants then you may want to use <see cref="Atan(decimal)"/></para>
		/// </summary>
		/// <param name="X">Tangent of the Angle</param>
		/// <returns>Angle in Radians in the range of -Pi/2 to Pi/2 inclusive.</returns>
		public static decimal Atan(double X)
		{
			decimal dX = (decimal)X;
			return Atan(dX);
		}

		/// <summary>
		/// <para>Computes the angle (in Radians) that resulted in this Tangent Value using Series Expansion.
		/// Thus computing the inverse tangent or arctan.</para>
		/// <para>If you want to cover all quadrants then you may want to use <see cref="Atan(decimal)"/></para>
		/// </summary>
		/// <param name="X">Tangent of the Angle</param>
		/// <returns>Angle in Radians in the range of -Pi/2 to Pi/2 inclusive.</returns>
		public static decimal Atan(int X)
		{
			decimal dX = X;
			return Atan(dX);
		}

		#endregion //Atan Methods

		#region Atan2 Methods

		/// <summary>
		/// Computes the angle (in Radians) that resulted in this Tangent value of Y / X using <see cref="Atan(decimal)"/>.
		/// Thus computing the inverse tangent or arctan.
		/// </summary>
		/// <param name="Y">Y Co-ordinate</param>
		/// <param name="X">X Co-ordinate</param>
		/// <returns>Angle in Radians in the range of -Pi (exclusive) to Pi (inclusive).</returns>
		public static decimal Atan2(decimal Y, decimal X)
		{

			#region Arguments Check

			if (CompareOps.ValueIsZero(X))
			{

				if (CompareOps.ValueIsZero(Y))
				{
					return 0.0m;
				}

				if (CompareOps.ValueIsPositive(Y))
				{
					return DecimalGlobals.PiOver2;
				}
				return -DecimalGlobals.PiOver2;
			}

			if (CompareOps.ValueIsZero(Y))
			{
				if (CompareOps.ValueIsPositive(X))
				{
					return 0.0m;
				}
				return DecimalGlobals.Pi;
			}

			#endregion

			decimal Z = Atan(Math.Abs(Y / X));

			if (CompareOps.ValueIsPositive(X))
			{
				if (CompareOps.ValueIsPositive(Y))
				{
					return Z;
				}
				return -Z;
			}
			if (CompareOps.ValueIsPositive(Y))
			{
				return DecimalGlobals.Pi - Z;
			}
			return -DecimalGlobals.Pi + Z;
		}

		/// <summary>
		/// Computes the angle (in Radians) that resulted in this Tangent value of Y / X using <see cref="Atan(decimal)"/>.
		/// Thus computing the inverse tangent or arctan.
		/// </summary>
		/// <param name="X">X Co-ordinate</param>
		/// <param name="Y">Y Co-ordinate</param>
		/// <returns>Angle in Radians in the range of -Pi (exclusive) to Pi (inclusive).</returns>
		public static decimal Atan2(double X, double Y)
		{
			decimal dX = (decimal)X;
			decimal dY = (decimal)Y;
			return Atan2(dX, dY);
		}

		/// <summary>
		/// Computes the angle (in Radians) that resulted in this Tangent value of Y / X using <see cref="Atan(decimal)"/>.
		/// Thus computing the inverse tangent or arctan.
		/// </summary>
		/// <param name="X">X Co-ordinate</param>
		/// <param name="Y">Y Co-ordinate</param>
		/// <returns>Angle in Radians in the range of -Pi (exclusive) to Pi (inclusive).</returns>
		public static decimal Atan2(int X, int Y)
		{
			decimal dX = X;
			decimal dY = Y;
			return Atan2(dX, dY);
		}

		#endregion //Atan2 Methods

		#region  Cosec Methods

		/// <summary>
		/// Computes the Cosecant of the given angle (in Radians) using <seealso cref="Sin(decimal)"/>.
		/// </summary>
		/// <param name="AngleInRadians">Angle in Radians</param>
		/// <returns>
		/// Cosecant of the Angle
		/// </returns>
		/// <exception cref="DivideByZeroException"><c>DivideByZeroException</c>.</exception>
		[System.ComponentModel.Description("Computes the Cosecant of the given angle (in Radians).")]
		public static decimal Cosec(decimal AngleInRadians)
		{
			decimal X = Sin(AngleInRadians);
			if (CompareOps.ValueIsZero(X))
			{
				throw new DivideByZeroException();
			}

			return 1 / X;
		}

		/// <summary>
		/// <para>Computes the Cosecant of the given angle (in Radians) using <seealso cref="Sin(decimal)"/>.</para>
		/// </summary>
		/// <param name="AngleInRadians">Angle in Radians</param>
		/// <returns>Cosecant of the Angle</returns>
		/// <exception cref="DivideByZeroException">
		/// Thrown when the angle falls on the 0 or Pi Axes.
		/// </exception>
		public static decimal Cosec(double AngleInRadians)
		{
			decimal X = Sin(AngleInRadians);
			if (CompareOps.ValueIsZero(X))
			{
				throw new DivideByZeroException();
			}

			return 1 / X;
		}

		/// <summary>
		/// <para>Computes the Cosecant of the given angle (in Radians) using <seealso cref="Sin(decimal)"/>.</para>
		/// </summary>
		/// <param name="AngleInRadians">Angle in Radians</param>
		/// <returns>Cosecant of the Angle</returns>
		/// <exception cref="DivideByZeroException">
		/// Thrown when the angle falls on the 0 or Pi Axes.
		/// </exception>
		public static decimal Cosec(int AngleInRadians)
		{
			decimal X = Sin(AngleInRadians);
			if (CompareOps.ValueIsZero(X))
			{
				throw new DivideByZeroException();
			}

			return 1 / X;
		}

		#endregion //Sec Methods

		#region  Sec Methods

		/// <summary>
		/// <para>Computes the Secant of the given angle (in Radians) using <seealso cref="Cos(decimal)"/>.</para>
		/// </summary>
		/// <param name="AngleInRadians">Angle in Radians</param>
		/// <returns>Secant of the Angle</returns>
		/// <exception cref="DivideByZeroException">
		/// Thrown when the angle falls on the Pi/2 or 3*Pi/2 Axes.
		/// </exception>
		public static decimal Sec(decimal AngleInRadians)
		{
			decimal X = Cos(AngleInRadians);
			if (CompareOps.ValueIsZero(X))
			{
				throw new DivideByZeroException();
			}

			return 1 / X;
		}

		/// <summary>
		/// <para>Computes the Secant of the given angle (in Radians) using <seealso cref="Cos(decimal)"/>.</para>
		/// </summary>
		/// <param name="AngleInRadians">Angle in Radians</param>
		/// <returns>Secant of the Angle</returns>
		/// <exception cref="DivideByZeroException">
		/// Thrown when the angle falls on the Pi/2 or 3*Pi/2 Axes.
		/// </exception>
		public static decimal Sec(double AngleInRadians)
		{
			decimal X = Cos(AngleInRadians);
			if (CompareOps.ValueIsZero(X))
			{
				throw new DivideByZeroException();
			}

			return 1 / X;
		}

		/// <summary>
		/// <para>Computes the Secant of the given angle (in Radians) using <seealso cref="Cos(decimal)"/>.</para>
		/// </summary>
		/// <param name="AngleInRadians">Angle in Radians</param>
		/// <returns>Secant of the Angle</returns>
		/// <exception cref="DivideByZeroException">
		/// Thrown when the angle falls on the Pi/2 or 3*Pi/2 Axes.
		/// </exception>
		public static decimal Sec(int AngleInRadians)
		{
			decimal X = Cos(AngleInRadians);
			if (CompareOps.ValueIsZero(X))
			{
				throw new DivideByZeroException();
			}

			return 1 / X;
		}

		#endregion //Sec Methods

		#region Cot Methods

		/// <summary>
		/// Computes the Cotangent of the given angle (in Radians).
		/// </summary>
		/// <param name="AngleInRadians">Angle in Radians</param>
		/// <returns>Cotangent of the Angle</returns>
		/// <exception cref="DivideByZeroException">
		/// Thrown when the angle falls on the 0 or Pi Axes.
		/// </exception>
		public static decimal Cot(decimal AngleInRadians)
		{
			decimal SinX;
			decimal CosX;
			SinCos(AngleInRadians, out SinX, out CosX);

			if (CompareOps.ValueIsZero(SinX))
			{
				throw (new DivideByZeroException());
			}
			return CosX / SinX;
		}

		/// <summary>
		/// Computes the Cotangent of the given angle (in Radians).
		/// </summary>
		/// <param name="AngleInRadians">Angle in Radians</param>
		/// <returns>Cotangent of the Angle</returns>
		/// <exception cref="DivideByZeroException">
		/// Thrown when the angle falls on the 0 or Pi Axes.
		/// </exception>
		public static decimal Cot(double AngleInRadians)
		{
			decimal dX = (decimal)AngleInRadians;
			return Cot(dX);
		}

		/// <summary>
		/// Computes the Cotangent of the given angle (in Radians).
		/// </summary>
		/// <param name="AngleInRadians">Angle in Radians</param>
		/// <returns>Cotangent of the Angle</returns>
		/// <exception cref="DivideByZeroException">
		/// Thrown when the angle falls on the 0 or Pi Axes.
		/// </exception>
		public static decimal Cot(int AngleInRadians)
		{
			decimal dX = AngleInRadians;
			return Cot(dX);
		}

		#endregion //Cot Methods

		#endregion //Radian Trig Ops

		#region Degree Trig Ops

		#region TanDeg Methods

		/// <summary>
		/// Computes the Tangent of the Angle, given the Angle in Degrees, using <see cref="Tan(decimal)"/>.
		/// </summary>
		/// <param name="AngleInDegrees">Angle in Degrees</param>
		/// <returns>Tangent of the Angle</returns>
		public static decimal TanDeg(decimal AngleInDegrees)
		{
			return Tan(Deg2Rad(AngleInDegrees));
		}

		/// <summary>
		/// Computes the Tangent of the Angle, given the Angle in Degrees, using <see cref="Tan(decimal)"/>.
		/// </summary>
		/// <param name="AngleInDegrees">Angle in Degrees</param>
		/// <returns>Tangent of the Angle</returns>
		public static decimal TanDeg(double AngleInDegrees)
		{
			return Tan(Deg2Rad(AngleInDegrees));
		}

		/// <summary>
		/// Computes the Tangent of the Angle, given the Angle in Degrees, using <see cref="Tan(decimal)"/>.
		/// </summary>
		/// <param name="AngleInDegrees">Angle in Degrees</param>
		/// <returns>Tangent of the Angle</returns>
		public static decimal TanDeg(int AngleInDegrees)
		{
			return Tan(Deg2Rad(AngleInDegrees));
		}

		#endregion //TanDeg Methods

		#region SinDeg Methods

		/// <summary>
		/// Computes the Sine of the Angle, given the Angle in Degrees, using <see cref="Sin(decimal)"/>.
		/// </summary>
		/// <param name="AngleInDegrees">Angle in Degrees</param>
		/// <returns>Sine of the Angle</returns>
		public static decimal SinDeg(decimal AngleInDegrees)
		{
			return Sin(Deg2Rad(AngleInDegrees));
		}

		/// <summary>
		/// Computes the Sine of the Angle, given the Angle in Degrees, using <see cref="Sin(decimal)"/>.
		/// </summary>
		/// <param name="AngleInDegrees">Angle in Degrees</param>
		/// <returns>Sine of the Angle</returns>
		public static decimal SinDeg(double AngleInDegrees)
		{
			return Sin(Deg2Rad(AngleInDegrees));
		}

		/// <summary>
		/// Computes the Sine of the Angle, given the Angle in Degrees, using <see cref="Sin(decimal)"/>.
		/// </summary>
		/// <param name="AngleInDegrees">Angle in Degrees</param>
		/// <returns>Sine of the Angle</returns>
		public static decimal SinDeg(int AngleInDegrees)
		{
			return Sin(Deg2Rad(AngleInDegrees));
		}

		#endregion //SinDeg Methods

		#region CosDeg Methods

		/// <summary>
		/// Computes the Cosine of the Angle, given the Angle in Degrees, using <see cref="Cos(decimal)"/>.
		/// </summary>
		/// <param name="AngleInDegrees">Angle in Degrees</param>
		/// <returns>Cosine of the Angle</returns>
		public static decimal CosDeg(decimal AngleInDegrees)
		{
			return Cos(Deg2Rad(AngleInDegrees));
		}

		/// <summary>
		/// Computes the Cosine of the Angle, given the Angle in Degrees, using <see cref="Cos(decimal)"/>.
		/// </summary>
		/// <param name="AngleInDegrees">Angle in Degrees</param>
		/// <returns>Cosine of the Angle</returns>
		public static decimal CosDeg(double AngleInDegrees)
		{
			return Cos(Deg2Rad(AngleInDegrees));
		}

		/// <summary>
		/// Computes the Cosine of the Angle, given the Angle in Degrees, using <see cref="Cos(decimal)"/>.
		/// </summary>
		/// <param name="AngleInDegrees">Angle in Degrees</param>
		/// <returns>Cosine of the Angle</returns>
		public static decimal CosDeg(int AngleInDegrees)
		{
			return Cos(Deg2Rad(AngleInDegrees));
		}

		#endregion //CosDeg Methods

		#region AtanDeg Methods

		/// <summary>
		/// Computes the Inverse Tangent returning the Angle in Degrees.
		/// Uses <see cref="Atan(decimal)"/>.
		/// </summary>
		/// <param name="X">Tangent of the Angle</param>
		/// <returns>Angle in Degrees</returns>
		public static decimal AtanDeg(decimal X)
		{
			return Rad2Deg(Atan(X));
		}

		/// <summary>
		/// Computes the Inverse Tangent returning the Angle in Degrees.
		/// Uses <see cref="Atan(decimal)"/>.
		/// </summary>
		/// <param name="X">Tangent of the Angle</param>
		/// <returns>Angle in Degrees</returns>
		public static decimal AtanDeg(double X)
		{
			decimal dX = (decimal)X;
			return AtanDeg(dX);
		}

		/// <summary>
		/// Computes the Inverse Tangent returning the Angle in Degrees.
		/// Uses <see cref="Atan(decimal)"/>.
		/// </summary>
		/// <param name="X">Tangent of the Angle</param>
		/// <returns>Angle in Degrees</returns>
		public static decimal AtanDeg(int X)
		{
			decimal dX = X;
			return AtanDeg(dX);
		}

		#endregion //AtanDeg Methods

		#region Atan2Deg Methods

		/// <summary>
		/// Computes the Inverse Tangent, where the Tangent is Y /X, returning the Angle in Degrees.
		/// Uses <see cref="Atan(decimal)"/>.
		/// </summary>
		/// <param name="Y">Y Co-ordinate</param>
		/// <param name="X">X Co-ordinate</param>
		/// <returns>Angle in Degrees</returns>
		public static decimal Atan2Deg(decimal Y, decimal X)
		{
			return Rad2Deg(Atan2(Y, X));
		}

		/// <summary>
		/// Computes the Inverse Tangent, where the Tangent is Y /X, returning the Angle in Degrees.
		/// Uses <see cref="Atan(decimal)"/>.
		/// </summary>
		/// <param name="Y">Y Co-ordinate</param>
		/// <param name="X">X Co-ordinate</param>
		/// <returns>Angle in Degrees</returns>
		public static decimal Atan2Deg(double Y, double X)
		{
			decimal dX = (decimal)X;
			decimal dY = (decimal)Y;
			return Atan2Deg(dY, dX);
		}

		/// <summary>
		/// Computes the Inverse Tangent, where the Tangent is Y /X, returning the Angle in Degrees.
		/// Uses <see cref="Atan(decimal)"/>.
		/// </summary>
		/// <param name="Y">Y Co-ordinate</param>
		/// <param name="X">X Co-ordinate</param>
		/// <returns>Angle in Degrees</returns>
		public static decimal Atan2Deg(int Y, int X)
		{
			decimal dX = X;
			decimal dY = Y;
			return Atan2Deg(dY, dX);
		}

		#endregion //Atan2Deg Methods

		#region AsinDeg Methods


		/// <summary>
		/// Computes the Inverse Sine returning the Angle in Degrees.
		/// Uses <see cref="Asin(decimal)"/>.
		/// </summary>
		/// <param name="X">Sine of the Angle</param>
		/// <returns>Angle in Degrees</returns>
		public static decimal AsinDeg(decimal X)
		{
			return Rad2Deg(Asin(X));
		}

		/// <summary>
		/// Computes the Inverse Sine returning the Angle in Degrees.
		/// Uses <see cref="Asin(decimal)"/>.
		/// </summary>
		/// <param name="X">Sine of the Angle</param>
		/// <returns>Angle in Degrees</returns>
		public static decimal AsinDeg(double X)
		{
			decimal dX = (decimal)X;
			return AsinDeg(dX);
		}

		#endregion //AsinDeg Methods

		#region AcosDeg Methods

		/// <summary>
		/// Computes the Inverse Cosine returning the Angle in Degrees.
		/// Uses <see cref="Acos(decimal)"/>.
		/// </summary>
		/// <param name="X">Cosine of the Angle</param>
		/// <returns>Angle in Degrees</returns>
		public static decimal AcosDeg(decimal X)
		{
			return Rad2Deg(Acos(X));
		}

		/// <summary>
		/// Computes the Inverse Cosine returning the Angle in Degrees.
		/// Uses <see cref="Acos(decimal)"/>.
		/// </summary>
		/// <param name="X">Cosine of the Angle</param>
		/// <returns>Angle in Degrees</returns>
		public static decimal AcosDeg(double X)
		{
			decimal dX = (decimal)X;
			return AcosDeg(dX);
		}

		#endregion //AcosDeg Methods


		#region  CosecDeg Methods

		/// <summary>
		/// <para>Computes the Cosecant of the given angle (in Degrees) using <seealso cref="Sin(decimal)"/>.</para>
		/// </summary>
		/// <param name="AngleInDegrees">Angle in Degrees</param>
		/// <returns>Cosecant of the Angle</returns>
		/// <exception cref="DivideByZeroException">
		/// Thrown when the angle falls on the 0 or Pi Axes.
		/// </exception>
		public static decimal CosecDeg(decimal AngleInDegrees)
		{
			return Cosec(Deg2Rad(AngleInDegrees));
		}

		/// <summary>
		/// <para>Computes the Cosecant of the given angle (in Degrees) using <seealso cref="Sin(decimal)"/>.</para>
		/// </summary>
		/// <param name="AngleInDegrees">Angle in Degrees</param>
		/// <returns>Cosecant of the Angle</returns>
		/// <exception cref="DivideByZeroException">
		/// Thrown when the angle falls on the 0 or Pi Axes.
		/// </exception>
		public static decimal CosecDeg(double AngleInDegrees)
		{
			return Cosec(Deg2Rad(AngleInDegrees));
		}

		/// <summary>
		/// <para>Computes the Cosecant of the given angle (in Degrees) using <seealso cref="Sin(decimal)"/>.</para>
		/// </summary>
		/// <param name="AngleInDegrees">Angle in Degrees</param>
		/// <returns>Cosecant of the Angle</returns>
		/// <exception cref="DivideByZeroException">
		/// Thrown when the angle falls on the 0 or Pi Axes.
		/// </exception>
		public static decimal CosecDeg(int AngleInDegrees)
		{
			return Cosec(Deg2Rad(AngleInDegrees));
		}

		#endregion //CosecDeg Methods

		#region  Sec Methods

		/// <summary>
		/// <para>Computes the Secant of the given angle (in Degrees) using <seealso cref="Cos(decimal)"/>.</para>
		/// </summary>
		/// <param name="AngleInDegrees">Angle in Degrees</param>
		/// <returns>Secant of the Angle</returns>
		/// <exception cref="DivideByZeroException">
		/// Thrown when the angle falls on the Pi/2 or 3*Pi/2 Axes.
		/// </exception>
		public static decimal SecDeg(decimal AngleInDegrees)
		{
			return Sec(Deg2Rad(AngleInDegrees));
		}

		/// <summary>
		/// <para>Computes the Secant of the given angle (in Degrees) using <seealso cref="Cos(decimal)"/>.</para>
		/// </summary>
		/// <param name="AngleInDegrees">Angle in Degrees</param>
		/// <returns>Secant of the Angle</returns>
		/// <exception cref="DivideByZeroException">
		/// Thrown when the angle falls on the Pi/2 or 3*Pi/2 Axes.
		/// </exception>
		public static decimal SecDeg(double AngleInDegrees)
		{
			return Sec(Deg2Rad(AngleInDegrees));
		}

		/// <summary>
		/// <para>Computes the Secant of the given angle (in Degrees) using <seealso cref="Cos(decimal)"/>.</para>
		/// </summary>
		/// <param name="AngleInDegrees">Angle in Degrees</param>
		/// <returns>Secant of the Angle</returns>
		/// <exception cref="DivideByZeroException">
		/// Thrown when the angle falls on the Pi/2 or 3*Pi/2 Axes.
		/// </exception>
		public static decimal SecDeg(int AngleInDegrees)
		{
			return Sec(Deg2Rad(AngleInDegrees));
		}

		#endregion //SecDeg Methods

		#region CotDeg Methods

		/// <summary>
		/// Computes the Cotangent of the given angle (in Degrees).
		/// </summary>
		/// <param name="AngleInDegrees">Angle in Degrees</param>
		/// <returns>Cotangent of the Angle</returns>
		/// <exception cref="DivideByZeroException">
		/// Thrown when the angle falls on the 0 or Pi Axes.
		/// </exception>
		public static decimal CotDeg(decimal AngleInDegrees)
		{
			return Cot(Deg2Rad(AngleInDegrees));
		}

		/// <summary>
		/// Computes the Cotangent of the given angle (in Degrees).
		/// </summary>
		/// <param name="AngleInDegrees">Angle in Degrees</param>
		/// <returns>Cotangent of the Angle</returns>
		/// <exception cref="DivideByZeroException">
		/// Thrown when the angle falls on the 0 or Pi Axes.
		/// </exception>
		public static decimal CotDeg(double AngleInDegrees)
		{
			return Cot(Deg2Rad(AngleInDegrees));
		}

		/// <summary>
		/// Computes the Cotangent of the given angle (in Degrees).
		/// </summary>
		/// <param name="AngleInDegrees">Angle in Degrees</param>
		/// <returns>Cotangent of the Angle</returns>
		/// <exception cref="DivideByZeroException">
		/// Thrown when the angle falls on the 0 or Pi Axes.
		/// </exception>
		public static decimal CotDeg(int AngleInDegrees)
		{
			return Cot(Deg2Rad(AngleInDegrees));
		}

		#endregion //CotDeg Methods


		#endregion //Degree Trig Ops

		#endregion //Trigonometry

	}
}
